import cv2 as cv
import numpy as np

env = {
    "face_detection_model": "./models/face_detection_yunet_2023mar.onnx",
    "face_recognition_model": "./models/face_recognition_sface_2021dec.onnx",
    "score_threshold": 0.9,
    "nms_threshold": 0.3,
    "scale": 1.0,
    "top_k": 5000,
    "ground_base_image": "./task1_input/egor.jpg",
}


def visualize(frame, faces, fps, recognizer, ground_base_feature):
    thickness = 2
    
    if faces[1] is not None:
        for idx, face in enumerate(faces[1]):
            face_align = recognizer.alignCrop(frame, face)
            face_feature = recognizer.feature(face_align)
            
            cosine_similarity_threshold = 0.363
            l2_similarity_threshold = 1.128

            cosine_score = recognizer.match(
                face_feature,
                ground_base_feature,
                cv.FaceRecognizerSF_FR_COSINE,
            )
            l2_score = recognizer.match(
                face_feature,
                ground_base_feature,
                cv.FaceRecognizerSF_FR_NORM_L2,
            )
    
            if cosine_score >= cosine_similarity_threshold or l2_score <= l2_similarity_threshold:
                color = (0, 255, 0)
            else: 
                color = (0, 0, 255)
            
            coords = face[:-1].astype(np.int32)
            cv.rectangle(
                frame, 
                (coords[0], coords[1]), 
                (coords[0] + coords[2], coords[1] + coords[3]), 
                color, 
                thickness,
            )
            
    cv.putText(
        frame,
        "FPS: {:.2f}".format(fps),
        (1, 16),
        cv.FONT_HERSHEY_SIMPLEX,
        0.5,
        (0, 255, 0),
        2,
    )


def get_ground_base_feature(detector, recognizer):
    ground_base_image = cv.imread(cv.samples.findFile(env["ground_base_image"]))

    width = int(ground_base_image.shape[1])
    height = int(ground_base_image.shape[0])
    detector.setInputSize((width, height))

    faces = detector.detect(ground_base_image)

    assert faces[1] is not None, "Cannot find a face"

    face_align = recognizer.alignCrop(ground_base_image, faces[1][0])
    face_feature = recognizer.feature(face_align)

    return face_feature


def main():
    detector = cv.FaceDetectorYN.create(
        env["face_detection_model"],
        "",
        (320, 320),
        env["score_threshold"],
        env["nms_threshold"],
        env["top_k"],
    )
    recognizer = cv.FaceRecognizerSF.create(
        env["face_recognition_model"],
        "",
    )

    tick_meter = cv.TickMeter()

    device_id = 0
    video_capture = cv.VideoCapture(device_id)
    
    frame_width = int(
        video_capture.get(cv.CAP_PROP_FRAME_WIDTH) * env["scale"]
    )
    frame_height = int(
        video_capture.get(cv.CAP_PROP_FRAME_HEIGHT) * env["scale"]
    )
    frame_shape = (frame_width, frame_height)
    detector.setInputSize(frame_shape)

    ground_base_feature = get_ground_base_feature(detector, recognizer)
    detector.setInputSize(frame_shape)

    while cv.waitKey(1) < 0:
        has_frame, frame = video_capture.read()
        if not has_frame:
            print('No frames grabbed!')
            break

        frame = cv.resize(frame, frame_shape)

        tick_meter.start()
        faces = detector.detect(frame)
        tick_meter.stop()

        visualize(frame, faces, tick_meter.getFPS(), recognizer, ground_base_feature)

        cv.imshow('Live', frame)
    
    cv.destroyAllWindows()
    

if __name__ == '__main__':
    main()
