import os

import cv2 as cv
import numpy as np
import pandas as pd
from deepface import DeepFace

photos_directory = "task3_1_input/photos"
labels_directory = "task3_1_input/labels"
video_directory = "task3_1_input/videos"
output_directory = "task3_1_output"
frame_interval = 50


def time_to_frame(time_str, fps):
    hours, minutes, seconds = map(float, time_str.split(":"))

    total_seconds = hours * 3600 + minutes * 60 + seconds
    frame_number = int(total_seconds * fps)
    return frame_number


def get_persons_in_frame(labels, frame):
    for interval in labels:
        if interval[0] <= frame <= interval[1]:
            return interval[2]
    return []


def main():
    for filename in os.listdir(video_directory):
        video_path = os.path.join(video_directory, filename)
        video_output_dir = os.path.join(output_directory, os.path.splitext(filename)[0])

        labels_path = os.path.join(labels_directory, os.path.splitext(filename)[0] + ".csv")

        video_capture = cv.VideoCapture(video_path)
        fps = video_capture.get(cv.CAP_PROP_FPS)

        df = pd.read_csv(labels_path)
        true_positive, false_positive = 0, 0
        
        frame_count = int(video_capture.get(cv.CAP_PROP_FRAME_COUNT))
        labels = []

        for index, row in df.iterrows():
            interval_from = time_to_frame(row["from"], fps)
            interval_to = time_to_frame(row["to"], fps)
            interval_persons = row["persons"].split(", ")
            labels.append([interval_from, interval_to, interval_persons])

        if not os.path.exists(video_output_dir):
            os.makedirs(video_output_dir)

        print("==============================")
        print("Video: ", filename)
        print("Labels: ", labels)
        print("Length: ", frame_count)

        current_frame_index = 0
        success, image = video_capture.read()

        if not success:
            print(f"Failed to read first frame of {filename}, may be it's broken. Skipping this video...")
            continue

        correct_frames = []
        while current_frame_index < frame_count:
            if image is None:
                break
            result_image = image.copy()
            np_image = np.asarray(image[:, :])
            print("Processing frame: ", current_frame_index)
            frame_filename = os.path.join(video_output_dir, f"frame_{current_frame_index}.jpg")

            try:
                detected_faces = DeepFace.extract_faces(np_image)
            except Exception as e:
                print(f"Failed to extract faces with exception {e}")
                detected_faces = []

            for face in detected_faces:
                x = face["facial_area"]["x"]
                y = face["facial_area"]["y"]
                x2 = x + face["facial_area"]["w"]
                y2 = y + face["facial_area"]["h"]
                cv.rectangle(result_image, (x, y), (x2, y2), (0, 0, 255), 2)

            labeled_persons = get_persons_in_frame(labels, current_frame_index)

            for labeled_person in labeled_persons:
                verified = False
                person_path = os.path.join(photos_directory, labeled_person + "/no_glasses.JPG")

                for face in detected_faces:
                    x = face["facial_area"]["x"]
                    y = face["facial_area"]["y"]
                    x2 = face["facial_area"]["w"] + x
                    y2 = face["facial_area"]["h"] + y

                    try:
                        verified = DeepFace.verify(person_path, np_image[y:y2, x:x2])["verified"]
                    except Exception as e:
                        print(f"Failed to verify person {person_path} with exception {e}")
                    if verified:
                        break

                if verified:
                    cv.rectangle(
                        result_image,
                        (x, y),
                        (x2, y2),
                        (0, 255, 0),
                        2,
                    )
                    cv.putText(
                        result_image,
                        labeled_person,
                        (x, y - 5),
                        cv.FONT_HERSHEY_SIMPLEX,
                        0.5,
                        (0, 0, 255),
                        2,
                    )
                    true_positive += 1
                    correct_frames.append(current_frame_index)
                    cv.imwrite(frame_filename, result_image)
                else:
                    false_positive += 1

            current_frame_index += frame_interval
            video_capture.set(cv.CAP_PROP_POS_FRAMES, current_frame_index)
            success, image = video_capture.read()

        print("True Positives: ", true_positive)
        print("False Positives: ", false_positive)
        print("Precision: ", float(true_positive) / (float(true_positive) + float(false_positive)))
        print("Correct Frames: ", correct_frames)

        with open(os.path.join(output_directory, "results.txt"), "a") as f:
            f.write("==============================\n")
            f.write(f"Video: {filename}\n")
            f.write(f"Labels: {labels}\n")
            f.write(f"Length: {frame_count}\n")
            f.write(f"True Positives: {true_positive}\n")
            f.write(f"False Positives: {false_positive}\n")
            f.write(f"Precision: {float(true_positive) / (float(true_positive) + float(false_positive))}\n")
            f.write(f"Correct Frames: {correct_frames}\n")

        video_capture.release()



if __name__ == "__main__":
    main()